/*
 *  Distances.h
 *  Keymileage
 *
 *  Created by Allen Smith on Tue Mar 02 2004.
 *
 */
#include <math.h>


static const char *unitNames[5][2] =
	{
		{"inch", "inches"},
		{"foot", "feet"},
		{"yard", "yards"},
		{"furlong", "furlongs"},
		{"mile", "miles"}
	};
	
#define INCHES		0
#define FEET		1
#define YARDS		2
#define FURLONGS	3
#define MILES		4

#define INCHES_PER_MILE (12*5280)
#define INCHES_PER_FURLONG (12*660)
#define INCHES_PER_FOOT 12
#define FEET_PER_MILE 5280

/**************************************************************************
 * extractMiles
 *	returns the integral number of miles these inches amount to
 **************************************************************************/
#define extractMiles(inches) (\
	( (int) floor( (inches)/INCHES_PER_MILE ) ) \
)

/**************************************************************************
 * extractFeet
 *	returns the number of feet in these inches as an integer
 *	no regard for spilling into miles
 **************************************************************************/
#define extractFeet(inches) (\
	(int) floor( (inches)/INCHES_PER_FOOT )\
)

/**************************************************************************
 * extractFeetUnderMiles
 *	returns the number of feet under a mile
 **************************************************************************/
#define extractFeetUnderMiles(inches) (\
	extractFeet(inches) % FEET_PER_MILE \
)

/**************************************************************************
 * extractInches
 *	returns the number of inches under a foot
 **************************************************************************/
#define extractInches(totalInches) (\
	fmod( (totalInches), INCHES_PER_FOOT) \
)

/**************************************************************************
 * plurality
 *	returns 1 if number is greater than 1
 *	or if number is equal to 0 (because we say 0 feet, not 0 foot)
 *	0 otherwise
 **************************************************************************/
#define plurality(number) (\
	((number) > 1 || (number) == 0) ? 1 : 0 \
)