#import "KeyboardView.h"

@implementation KeyboardView

/**************************************************************************
 * initWithFrame
 *	sets up the bounds of the view
 **************************************************************************/
- (id)initWithFrame:(NSRect)frameRect
{
	if ((self = [super initWithFrame:frameRect]) != nil) {
		//printf("origin: %f,%f; size: %f,%f\n", frameRect.origin.x, frameRect.origin.y, frameRect.size.width, frameRect.size.height);
		viewScaleFactor = 8;
	}
	return self;
}


/**************************************************************************
 * drawRect
 *	draws all the keys up on the screen!
 **************************************************************************/
- (void)drawRect:(NSRect)rect{
	int keycode;
	int row;
	float offset, keyWidth, keyHeight, rowHeight;
	NSPoint keyCapPosition;
	NSSize keyCapDimensions;
	rowHeight = keyboard->getRowHeight();
	keyHeight = keyboard->getKeyHeight();
	char letterLabel[] = {'\0', '\0', '\0'};
	
	//Font Attributes for keyCaps
	NSMutableDictionary *attributes = [NSMutableDictionary dictionary];
	NSFont				*font		= [NSFont boldSystemFontOfSize:4.5];
	
	[attributes setObject:font
				forKey:NSFontAttributeName];
	[attributes setObject:[NSColor whiteColor]
				forKey:NSForegroundColorAttributeName];
	
	NSRect bounds = [self bounds];
	[[NSColor whiteColor] set];
	NSRectFill(bounds);
	
	for(keycode = 0; keycode < NUMBER_KEYCODES; keycode++){
		Key *thisKey = keyboard->getKey(keycode);
		keyWidth = thisKey->getKeyWidth();
		offset = thisKey->getOffset();
		row = thisKey->getRow();
			
		//Create the Key Rectangle
		NSRect keyRectangle = [self makeKeyboardNSRect: NSMakePoint(offset, row*rowHeight+keyHeight)
													dimensions: NSMakeSize(keyWidth,keyHeight) ];
		[[NSColor darkGrayColor] set];
		NSRectFill(keyRectangle);
		
		//Print the letter label on the key
		letterLabel[0] = keyboardLayout->getCharacterForKeycode(keycode);
		//sprintf(letterLabel,"%d",keycode); //use this to display keycodes instead
		if(letterLabel[0] != '\0'){
			NSString *keyCap = [NSString stringWithCString:letterLabel];
			keyCapDimensions = [keyCap sizeWithAttributes:attributes];
			//Center the keyCap label on the key itself
			keyCapPosition.x = offset + keyWidth/2 - (keyCapDimensions.width/viewScaleFactor)/2;
			keyCapPosition.y = row*rowHeight + keyHeight/2 + (keyCapDimensions.height/viewScaleFactor)/2;
			[keyCap drawAtPoint: [self keyboardPointToView:keyCapPosition]
					withAttributes:attributes];
		}
	}

}

/**************************************************************************
 * setKeyboard
 *	sets the specified keyboards for drawing.
 **************************************************************************/
- (void)setKeyboard:(Keyboard *)keyboardToDraw layout:(KeyboardLayout *)keyboardLayoutToDraw{
	Dimensions keyboardDimensions;
	keyboard = keyboardToDraw;
	keyboardLayout = keyboardLayoutToDraw;
	keyboardDimensions = keyboard->getKeyboardDimensions();
	keyboardHeight = keyboardDimensions.y;
	[self setBounds: NSMakeRect(0,0, keyboardDimensions.x * viewScaleFactor, keyboardHeight * viewScaleFactor)];
	[self setNeedsDisplay: YES];
}


/**************************************************************************
 * keyboardPointToView
 *	converts keyboard coordinates into view coordinates.
 **************************************************************************/
- (NSPoint)keyboardPointToView:(NSPoint)keyboardPoint{
	NSPoint viewPoint;
	viewPoint.x = keyboardPoint.x * viewScaleFactor;
	viewPoint.y = (keyboardHeight - keyboardPoint.y) * viewScaleFactor;
	return viewPoint;
}


/**************************************************************************
 * keyboardPointToView
 *	converts keyboard coordinates into view coordinates.
 **************************************************************************/
- (NSPoint)keyboardPointToView:(float)x y:(float)y{
	NSPoint viewPoint;
	viewPoint.x = x * viewScaleFactor;
	viewPoint.y = (keyboardHeight - y) * viewScaleFactor;
	return viewPoint;
}


/**************************************************************************
 * makeKeyboardNSRect
 *	converts keyboard coordinates into view coordinates
 *  returns rectangle with the specified width and height
 **************************************************************************/
- (NSRect)makeKeyboardNSRect:(NSPoint)origin dimensions:(NSSize)dimensions{
	NSPoint viewOrigin = [self keyboardPointToView:origin];
	return NSMakeRect(viewOrigin.x, viewOrigin.y, dimensions.width * viewScaleFactor, dimensions.height * viewScaleFactor);
}

@end
