#import "ProgressDisplay.h"

@implementation ProgressDisplay

/**************************************************************************
 * updateIfNeededTo
 *	if appropriate, adjusts the progress indicator
 *  returns YES if the progress was updated
 **************************************************************************/
- (BOOL)updateIfNeededTo:(double)newValue{
	BOOL didUpdate = NO;
	if( -[timeOfLastUpdate timeIntervalSinceNow] > secondsPerUpdate){ //timeIntervalSinceNow returns negative numbers!
		[progressBar setDoubleValue:newValue];
		[progressBar displayIfNeeded];
		timeOfLastUpdate = [NSDate date]; //but, isn't this a memory leak?! If you try to release it first, the program crashes though. Huh.
		//Actually, no leak. Objects I get this way are autoreleased.
		didUpdate = YES;
	}
	if([progressBarWindow isVisible] == NO)
		if( -[startTime timeIntervalSinceNow] > startProgressBarAt){
			//Show the progress bar
			progressModalSession = [NSApp beginModalSessionForWindow:progressBarWindow];
			[NSApp runModalSession:progressModalSession];
			//[progressBarWindow makeKeyAndOrderFront:self]; //use if we don't want a really annoying modal session
		}
	return didUpdate;
}

/**************************************************************************
 * openAsIndeterminate
 *	displays the progress bar window, but as an interterminate progress
 **************************************************************************/
- (void)openIndeterminateWithModality:(BOOL)runAsModalSession{
	[progressBar setIndeterminate:YES];
	if(runAsModalSession == YES){
		progressModalSession = [NSApp beginModalSessionForWindow:progressBarWindow];
		[NSApp runModalSession:progressModalSession];
	}
	else
		[progressBarWindow makeKeyAndOrderFront:self];
}

/**************************************************************************
 * startWithMaximum
 *	changes bar to determinate and sets the maximum value the progress bar can achieve
 **************************************************************************/
- (void)startWithMaximum:(double)newMaximum{
	secondsPerUpdate = 0.1666;
	startProgressBarAt = 0.25;
	startTime = [NSDate date]; //the current time
	timeOfLastUpdate = [NSDate date];
	[progressBar setIndeterminate:NO];
	[progressBar setMaxValue:newMaximum];
}

/**************************************************************************
 * endUpdating
 *	removes the progress bar window
 **************************************************************************/
- (void)endUpdating{
	//remove progress bar window
	if([progressBarWindow isVisible]){
		[progressBarWindow close];
	}
	if([NSApp modalWindow] != nil) //modalWindow returns nil if there is no modal session in progress
		[NSApp endModalSession:progressModalSession];
}


@end
