/*
 *  finger.cpp
 *  Keymileage
 *
 *	Class to track information for a single finger in a hand
 *
 *  Created by Allen Smith on Thu Feb 26 2004.
 *
 */

#include "finger.h"
#include <math.h>
#include <iostream>
using std::cout;
using std::endl;

/*Default Constructor*/
Finger::Finger(){
	totalInchesTraveled = 0;
}

/**********************************************
 * Finger
 *	constructor
 **********************************************/
Finger::Finger(Coordinate defaultPoint){
	defaultPosition = defaultPoint;
	currentPosition = defaultPoint;
	strokesTillDefault = 0;
	totalInchesTraveled = 0;
	totalStrokes = 0;
}


/**********************************************
 * decrementStrokeCounter
 *	signals that a key other than this one has been typed.
 *	finger is moved back to default position
 *	when this counter reaches 0
 *	returns
 *		the distance the finger travels if it is reset
 *		0 otherwise
 **********************************************/
float Finger::decrementStrokeCounter(){
	float distanceMoved = 0;
	
	if(strokesTillDefault > 0){
		strokesTillDefault--;
		if(strokesTillDefault == 0)
			distanceMoved = moveFinger(&defaultPosition);
	}
	
	return distanceMoved;
}


/**********************************************
 * moveFinger
 *	sends a finger to the coordinate specified
 **********************************************/
float Finger::moveFinger(Coordinate *newPoint){
	float distanceMoved = 0;

#ifdef DEBUG_FINGERS	
	cout << "Moving finger from "
		 << "(" << currentPosition.x << "," << currentPosition.y << ")"
		 << " to "
		 << "(" << newPoint->x << "," << newPoint->y << ")"
		 << endl;
#endif
	
	//THE DISTANCE ALGORITHM
	//currently just the distance formula
	//this is only measuring linear distance
	distanceMoved = sqrt(	pow( newPoint->x - currentPosition.x ,2) +
							pow( newPoint->y - currentPosition.y ,2)   );
	currentPosition = *newPoint;
	totalInchesTraveled += distanceMoved; //update the finger distance
	return distanceMoved;
}


/**********************************************
 * typeFinger
 *	sends a finger to the coordinate specified
 *	to strike a letter
 **********************************************/
float Finger::typeFinger(Coordinate *newPoint){
	float distanceMoved = 0;
	
	distanceMoved = moveFinger(newPoint);
//put in some code for distance it takes to actually *press* the key
	if(distanceMoved > 0) //only worry about stroke reset flags if the finger actually moved somewhere.
		strokesTillDefault = numberStrokesTillDefault;
	totalStrokes++;
//cout<<"moved to ("<<newPoint->x <<","<< newPoint->y << ")" <<endl;
	return distanceMoved;
}


/**********************************************
 * getInchesTraveled
 *	returns the total distance this finger has traveled
 **********************************************/
double Finger::getInchesTraveled(){
	return totalInchesTraveled;
}


/**********************************************
 * getCurrentPosition
 *	returns the total distance this finger has traveled
 **********************************************/
Coordinate Finger::getCurrentPosition(){
	return currentPosition;
}


/**********************************************
 * forceReturnToDefault
 *	sends the finger back to its default position
 *	whether or not it is scheduled to go there.
 **********************************************/
float Finger::forceReturnToDefault(){
	float distanceMoved = 0;
	
	strokesTillDefault = 0;
	distanceMoved = moveFinger(&defaultPosition);
	
	return distanceMoved;
}


/**********************************************
 * resetFinger
 *	clears out finger distances
 *	should be called when a new file is to be analyzed
 **********************************************/
void Finger::resetFinger(){
	currentPosition = defaultPosition;
	strokesTillDefault = 0;
	totalInchesTraveled = 0;
}