/*
 *  hands.cpp
 *  Keymileage
 *
 *  Created by Allen Smith on Thu Feb 26 2004.
 *
 */

#include "hands.h"

#include <iostream>
using std::cout;
using std::endl;


//a global variable; especially needs to be seen by Finger objects
int numberStrokesTillDefault;


/**********************************************
 * Hands
 *	constructor
 **********************************************/
Hands::Hands( Keyboard *keyboardUsed, KeyboardLayout *keyboardLayoutBeingTyped){
	numberStrokesTillDefault = 3;
	keyboard = keyboardUsed;
	keyboardLayout = keyboardLayoutBeingTyped;
	distanceTyped = 0;
	int spacebarKeycode;
	
	//Left Hand
	for(int counter = 0; counter < 4; counter++)
		finger[counter] = *new Finger( *keyboard->getKeyByRowPosition(2,counter).getStrikepoint() );
	//Right Hand
		//First: The Thumb
	spacebarKeycode = keyboardLayout->getKeycodeForCharacter(' ');
	finger[5] = *new Finger( *keyboard->getKey(spacebarKeycode)->getStrikepoint() );
	for(int counter = 6; counter < 10; counter++)
		finger[counter] = *new Finger( *keyboard->getKeyByRowPosition(2,counter).getStrikepoint() );
}


/**********************************************
 * getFinger
 *	returns the finger at the specifed index
 **********************************************/
Finger Hands::getFinger(int index){
	return finger[index];
}


/**********************************************
 * getDistance
 *	returns the total distance these fingers have typed
 **********************************************/
double Hands::getDistance(){
	return distanceTyped;
}


/**********************************************
 * returnToHomeRow
 *	sends all fingers to default positions
 **********************************************/
float Hands::returnToHomeRow(){
	float distanceMoved = 0;
	for(int counter = 0; counter < 10; counter++){
		distanceMoved += finger[counter].forceReturnToDefault();
	}
	distanceTyped += distanceMoved;
	return distanceMoved;
}


/**********************************************
 * resetHands
 *	returns all fingers to default positions
 *	and clears distances
 **********************************************/
void Hands::resetHands(){
	distanceTyped = 0;
	for(int counter = 0; counter < 10; counter++){
		finger[counter].resetFinger();
	}
}


/**********************************************
 * typeCharacter
 *	moves the appropriate finger to the key of
 *	the character specified.
 *	returns the distance the finger moves
 **********************************************/
float Hands::typeCharacter(unsigned char ascii){
	//Regular Key variables
	int keycode;
	Coordinate *strikePoint;
	int fingerIndex;
	//Shift Key variables
	bool shifted;
	int shiftKeycode;
	int shiftKeyIndex;
	int shiftFingerIndex;
	Coordinate *shiftStrikePoint;
	
	float distanceMoved = 0;
	
	//Deal with a Unix Linefeed (LF) character:
	if(ascii == 10)
		ascii = 13; //convert it to a Macintosh carriage return (CR).
	//Note that Mac OS X natively uses UNIX linefeeds.
	//However, the AppleWorks spreadsheet I used to generate the ascii table
	//still returns a CR for a lineending. (Does Carbon do CRs and Cocoa do LFs?)
	
	keycode = keyboardLayout->getKeycodeForCharacter(ascii);
	fingerIndex = keyboard->getKey(keycode)->getFinger();
	strikePoint = keyboard->getKey(keycode)->getStrikepoint();
	shifted = keyboardLayout->isShifted(ascii);

#ifdef DEBUG_FINGERS
	cout<<"Typing character "<<ascii <<" ("<<(int)ascii<< ")" <<endl;
#endif
	//first deal with the shift key
	if(shifted){
		shiftKeyIndex = (fingerIndex/5 + 1)%2; //returns the shift key of the opposite hand to the letter's finger
		shiftFingerIndex = shiftKeyIndex * 9; //either the 0 or 9 finger
		shiftKeycode = keyboardLayout->getShiftKeycode(shiftKeyIndex);
		shiftStrikePoint = keyboard->getKey(shiftKeycode)->getStrikepoint();
#ifdef DEBUG_FINGERS
	cout<<"Shift Key "<<shiftKeyIndex<<" pressed. Finger "<< shiftFingerIndex << " ";
#endif
		distanceMoved += finger[shiftFingerIndex].typeFinger(shiftStrikePoint);
	}
	
	//Now move the finger (updating other fingers too) and record the results
#ifdef DEBUG_FINGERS
	cout<<"Finger "<<fingerIndex <<" ";
#endif
	distanceMoved = finger[fingerIndex].typeFinger(strikePoint);
	for(int counter = 0; counter < 10; counter++){
		if(shifted && counter != shiftFingerIndex && counter != fingerIndex)
			distanceMoved += finger[counter].decrementStrokeCounter();
		else if(counter != fingerIndex)
			distanceMoved += finger[counter].decrementStrokeCounter();
	}
	distanceTyped += distanceMoved;
#ifdef DEBUG_FINGERS
	cout<<endl;	
#endif
	return distanceMoved;
}


