/*
 *
 * key.cpp
 *
 *	used to define the dimensions and code of a key on the keyboard.
 *	Independent of a letter.
 */

#include "key.h"

/**********************************************
 * Key
 *	non-functional default constructor
 **********************************************/
Key::Key(){}

/**********************************************
 * Key
 *	constructor for normal keys
 **********************************************/
Key::Key(int rowSpecified,
		 int positionSpecified,
		 int keycodeSpecified,
		 int fingerSpecified,
		 float offsetSpecified,
		 float keyWidthSpecified,
		 Coordinate strikepointSpecified ){
	keycode = keycodeSpecified;
	finger = fingerSpecified;
	row = rowSpecified;
	position = positionSpecified;
	offset = offsetSpecified;
	keyWidth = keyWidthSpecified;
	strikepoint = strikepointSpecified;
}


/**********************************************
 * Key
 *	constructor for absolutely-positioned keys
 **********************************************/
Key::Key(int rowSpecified,
		 float offsetSpecified,
		 float keyWidthSpecified,
		 Coordinate strikepointSpecified,
		 int keycodeSpecified,
		 int fingerSpecified ){
	keycode = keycodeSpecified;
	strikepoint = strikepointSpecified;
	offset = offsetSpecified;
	keyWidth = keyWidthSpecified;
	finger = fingerSpecified;
	row = rowSpecified;
	
	position = -1;
}


/**********************************************
 * getCoordinates
 *	returns a Coordinate struct with x and y position in it
 **********************************************/
Coordinate* Key::getStrikepoint(){
	return &strikepoint;
}


/**********************************************
 * setCoordinates
 *	accepts x and y values (in inches) specifying the center of the key
 *	(0,0) is the upper left of the keyboard.
 **********************************************/
void Key::setStrikepoint(float x, float y){
	strikepoint.x = x;
	strikepoint.y = y;
}


/**********************************************
 * getFinger
 *	returns the ID of the finger which is used to type this key
 *	0 is the left pinkie; 1 is the left ring finger; 9 is the right pinkie, etc.
 **********************************************/
int Key::getFinger(){
	return finger;
}

/**********************************************
 * getKeycode
 *	returns the keycode for this key
 *	who knows why one would ever need this function.
 **********************************************/
int Key::getKeycode(){
	return keycode;
}

/**********************************************
 * getKeyWidth
 *	returns the width of this key in inches
 **********************************************/
float Key::getKeyWidth(){
	return keyWidth;
}

/**********************************************
 * getOffset
 *	returns the distance from
 *	this key's left edge to the left edge of the keyboard
 **********************************************/
float Key::getOffset(){
	return offset;
}

/**********************************************
 * getPosition
 *	returns the number of keys to the left of this one. Non-standard sized keys don't count
 *	who knows why one would ever need this function.
 **********************************************/
int Key::getPosition(){
	return position;
}

/**********************************************
 * getRow
 *	returns the row number in which this key resides
 *	0 is the number row
 **********************************************/
int Key::getRow(){
	return row;
}