/*
 *
 * keyboard.cpp
 *
 *	organizes all the keys on a keyboard.
 *
 */

#include "keyboard.h"
#include <iostream>
#include <fstream.h>
#include <string.h>
#include <stdlib.h>

/**********************************************
 * Keyboard
 *	constructor
 **********************************************/
Keyboard::Keyboard(const char *fileName){
	ifstream *keyboardFile; //Objective-C++ under gcc 3.3 is incable of using this object unless it is allocated to a pointer
	char inputLine[256];
	char commentCharacter[] = "#"; //indicates a line of comments
	char delimiterCharacter[] = ":"; //delimits fields in lines of data
	char *decommentedString; //for tokenization
	char *delimitedString; //for tokenization

	//open the file
	keyboardFile = new ifstream(fileName);
	
	while( !keyboardFile->eof() ){
	keyboardFile->getline(inputLine, 256);
		if(inputLine[0] != '#'){ //scan past commented lines
			decommentedString = strtok(inputLine, commentCharacter); //obliterate endline comments
				if(decommentedString != NULL){
			
			//Obtain constants from file
				if(strstr(decommentedString, "ROWHEIGHT") != NULL)
					rowHeight = parseFloat(decommentedString);
				else if(strstr(decommentedString, "KEYHEIGHT") != NULL)
					keyHeight = parseFloat(decommentedString);
				else if(strstr(decommentedString, "KEYWIDTH") != NULL)
					keyWidth = parseFloat(decommentedString);
				else if(strstr(decommentedString, "KEYSPACING") != NULL)
					keySpacing = parseFloat(decommentedString);
				else if(strstr(decommentedString, "KEYBOARDWIDTH") != NULL)
					keyboardWidth = parseFloat(decommentedString);
				else if(strstr(decommentedString, "KEYBOARDHEIGHT") != NULL)
					keyboardHeight = parseFloat(decommentedString);
				
			//Get Row offsets
				else if(strstr(decommentedString, "Row") != NULL){
					int rowNumber;
					delimitedString = strtok(decommentedString, delimiterCharacter);
					rowNumber = parseInt(delimitedString);
					delimitedString = strtok(NULL, delimiterCharacter); //get next field
					rowOffset[rowNumber] = parseFloat(delimitedString);
				//	cout << "Row " << rowNumber << " is offset " << rowOffset[rowNumber] << endl;
				}
				
			//Read Absolutely-positioned key
				else if(strstr(decommentedString, "KeyAbsolute") != NULL){
					int rowNumber;
					float offset;
					float keyWidth;
					float strikePointX; //x-coordinate of the strikepoint
					int keycode;
					int finger;
					
					delimitedString = strtok(decommentedString, delimiterCharacter);
					rowNumber = parseInt(delimitedString);
					delimitedString = strtok(NULL, delimiterCharacter); //get next field
					offset = parseFloat(delimitedString);
					delimitedString = strtok(NULL, delimiterCharacter); //get next field
					keyWidth = parseFloat(delimitedString);
					delimitedString = strtok(NULL, delimiterCharacter); //get next field
					strikePointX = parseFloat(delimitedString);
					delimitedString = strtok(NULL, delimiterCharacter); //get next field
					keycode = parseInt(delimitedString);
					delimitedString = strtok(NULL, delimiterCharacter); //get next field
					finger = parseInt(delimitedString);
					
					Coordinate strikepoint;
					strikepoint.x = strikePointX;
					strikepoint.y = rowHeight * rowNumber + 0.5*rowHeight;

					Key thisKey(rowNumber, offset, keyWidth, strikepoint, keycode, finger);
					key[keycode] = thisKey;
				/*	
					cout << "KeyAbsolute "  << key[keycode].getRow() << ":"
											<< key[keycode].getOffset() << ":"
											<< key[keycode].getKeyWidth() << ":"
											<< key[keycode].getStrikepoint()->x << ":"
											<< key[keycode].getKeycode() << ":"
											<< key[keycode].getFinger() << endl;
				*/	
					} //end else KeyAbsolute
				
			//Read standard-sized key
				else if(strstr(decommentedString, "Key") != NULL){
					int rowNumber;
					int position;
					int keycode;
					int finger;
					
					delimitedString = strtok(decommentedString, delimiterCharacter);
					rowNumber = parseInt(delimitedString);
					delimitedString = strtok(NULL, delimiterCharacter);
					position = parseInt(delimitedString);
					delimitedString = strtok(NULL, delimiterCharacter);
					keycode = parseInt(delimitedString);
					delimitedString = strtok(NULL, delimiterCharacter);
					finger = parseInt(delimitedString);
					
					row[rowNumber][position] = keycode;
					Coordinate strikepoint;
					float offset;
					offset = rowOffset[rowNumber] + position*keyWidth;
					if(rowOffset[rowNumber] > 0)
						offset += (position+1)*keySpacing;
					else
						offset += (position)*keySpacing;
					strikepoint.x = offset + 0.5*keyWidth;
					strikepoint.y = rowHeight * rowNumber + 0.5*rowHeight;
					Key thisKey(rowNumber, position, keycode, finger, offset, keyWidth, strikepoint);
					key[keycode] = thisKey;
				/*	
					cout << "Key "  << key[keycode].getRow() << ":"
									<< key[keycode].getStrikepoint()->x << ":" 
									<< key[keycode].getKeycode() << ":" 
									<< key[keycode].getFinger() << endl;
				*/
				} //end if standard key
			}
		} //end if uncommented
	}//end while
} //end constructor


/**********************************************
 * getKey
 *	returns the key with the specified keycode
 **********************************************/
Key * Keyboard::getKey(int keycode){
	return &key[keycode];
}


/**********************************************
 * getKeyByRowPosition
 *	returns the key at the specied row and position
 **********************************************/
Key Keyboard::getKeyByRowPosition(int rowNumber, int positionNumber){
	return key[ row[rowNumber][positionNumber] ];
}


/**********************************************
 * getKeyHeight
 *	returns the height of each key; all keys are assumed to be the same height
 **********************************************/
float Keyboard::getKeyHeight(){
	return keyHeight;
}


/**********************************************
 * getRowHeight
 *	returns the height of a row of keys (this includes spacing)
 **********************************************/
float Keyboard::getRowHeight(){
	return rowHeight;
}


/**********************************************
 * getKeyboardDimensions
 *	returns the height of a row of keys (this includes spacing)
 **********************************************/
Dimensions Keyboard::getKeyboardDimensions(){
	Dimensions keyboardDimensions;
	keyboardDimensions.x = keyboardWidth;
	keyboardDimensions.y = keyboardHeight;
	return keyboardDimensions;
}


