/*
 *  keyboardLayout.cpp
 *  Keymileage
 *
 *  Created by Allen Smith on Tue Feb 24 2004.
 *  Copyright (c) 2004 __MyCompanyName__. All rights reserved.
 *
 */

#include "keyboardLayout.h"
#include "standardFunctions.h"
#include <iostream.h>
#include <fstream.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>

/**********************************************
 * KeyboardLayout
 *	constructor
 **********************************************/
KeyboardLayout::KeyboardLayout(const char * fileName){
	char inputLine[256];
	int  keycode;
	unsigned char ascii;
	char commentCharacter[] = "!"; //indicates a line of comments in an xmodmap table
	char assignmentCharacter[] = "="; //indicates a line of comments
	char delimiterCharacter[] = " "; //delimits fields in lines of data
	char *decommentedString; //for tokenization
	char *rightHandSide; //the right hand side string of assignments (like "keycode 8 = a A" -> " a A")
	char *delimitedString; //for tokenization
		
	
	int counter;
	for(counter = 0; counter < 256; counter++){
		keycodes[counter] = 0; //can't use memset to clear array! It stops after 64 sets.
		characters[counter] = 0;
	}
	
	//open the file
	ifstream keyboardLayoutFile(fileName);
	
	while( !keyboardLayoutFile.eof() ){
		keyboardLayoutFile.getline(inputLine, 256);
		if(inputLine[0] != '#'){ //scan past commented lines
			decommentedString = strtok(inputLine, commentCharacter); //obliterate endline comments

			//Now extract meaning from the file!
			if(decommentedString != NULL && strstr(decommentedString, "keycode") != NULL){
				rightHandSide = strtok(decommentedString, assignmentCharacter);
				//actually, it's the left hand side at the moment!
				keycode = parseInt(rightHandSide);
				
				rightHandSide = strtok(NULL, assignmentCharacter); //tokenizing again ensures that this is actually the RIGHT hand side, not the left
				//Now that we have the real right hand side, delimit it
				delimitedString = strtok(rightHandSide, delimiterCharacter);
				if(delimitedString != NULL && keycode < 256 ){
					if(strlen(delimitedString) == 1){ //it is a plain ASCII character
						ascii = delimitedString[0];
					}
					else if(strstr(delimitedString, "NoSymbol") == NULL &&
							strlen(delimitedString) > 0  ){
						ascii = xmodmapStringToASCII(delimitedString);
					}
					
					if(keycodes[ascii] == 0){ //only assign if it hasn't already been assigned!
						keycodes[ascii] = keycode;
						characters[keycode] = ascii;
						shifted[ascii] = false; //make sure to set this, because the array isn't otherwise initialized
					}
					//Maybe it's a modifier key. We want to remember some of them
					else if(strstr(delimitedString, "Shift_L"))
						shiftKey[0] = keycode;
					else if(strstr(delimitedString, "Shift_R"))
						shiftKey[1] = keycode;
					else if(strstr(delimitedString, "Caps_Lock"))
						capsLock = keycode;
					
					
					//Parse the shifted version of this key	
					delimitedString = strtok(NULL, delimiterCharacter);
					if(delimitedString != NULL && strstr(delimitedString, "NoSymbol") == NULL){
						if(strlen(delimitedString) == 1) //it is a plain ASCII character
							ascii = delimitedString[0];
						else
							ascii = xmodmapStringToASCII(delimitedString);
						
						if(keycodes[ascii] == 0){ //only assign if it hasn't already been assigned!
							keycodes[ascii] = keycode;
							if(isalpha(ascii)) //we want uppercase labels for letter keys, but not the shifted equivalent for anything else
								characters[keycode] = ascii;
							shifted[ascii] = true;
						}
					}//end shifted determiner
				}//end if for parsing key characters
			}//end if keycode
		}
	}//end while
	
} //end constructor


/**********************************************
 * getCharacterForKeycode
 *	returns the keycode associated with an ASCII char
 **********************************************/
unsigned char KeyboardLayout::getCharacterForKeycode(int keycode){
	return characters[keycode]; //the keycode
}


/**********************************************
 * getKeycodeForCharacter
 *	returns the keycode associated with an ASCII char
 **********************************************/
int KeyboardLayout::getKeycodeForCharacter(unsigned char ascii){
	return keycodes[ascii]; //the keycode
}


/**********************************************
 * isShifted
 *	returns whether or not the specified character
 *	is produced by holding down shift
 **********************************************/
bool KeyboardLayout::isShifted(unsigned char ascii){
	return shifted[ascii];
}


/**********************************************
 * getShiftKeycode
 *	returns whether or not the specified character
 *	is produced by holding down shift
 **********************************************/
int KeyboardLayout::getShiftKeycode(int shiftIndex){
	return shiftKey[shiftIndex];
}


/**********************************************
 * xmodmapStringToASCII
 *	returns the ASCII equivalent of the
 *	descriptive strings used in xmodmap
 **********************************************/
unsigned char KeyboardLayout::xmodmapStringToASCII(char * stringToResolve){
	static int numberOfSymbols = 99;
	//The tables, sorted lexigraphically:
	static char *xmodmapStrings[] = {
		"Aacute", "Acircumflex", "AE", "ae", "ampersand", "apostrophe", "Aring", "aring", "asciicircum", "asciitilde", "asterisk", "at", "backslash", "bar", "braceleft", "braceright", "bracketleft", "bracketright", "Ccedilla", "ccedilla", "cent", "colon", "comma", "copyright", "currency", "dagger", "dead_abovedot", "dead_abovering", "dead_acute", "dead_breve", "dead_caron", "dead_cedilla", "dead_circumflex", "dead_diaeresis", "dead_doubleacute", "dead_grave", "dead_macron", "dead_ogonek", "dead_tilde", "division", "dollar", "doubledagger", "doublelowquotemark", "ellipsis", "emdash", "endash", "equal", "exclam", "exclamdown", "grave", "greater", "guillemotleft", "guillemotright", "Iacute", "Icircumflex", "Idiaeresis", "idotless", "leftanglebracket", "leftdoublequotemark", "less", "masculine", "minus", "mu", "notsign", "numbersign", "Oacute", "Ocircumflex", "OE", "oe", "Ograve", "Ooblique", "ordfeminine", "oslash", "paragraph", "parenleft", "parenright", "percent", "period", "periodcentered", "plus", "plusminus", "question", "questiondown", "quotedbl", "registered", "Return", "rightdoublequotemark", "rightsinglequotemark", "section", "semicolon", "singlelowquotemark", "slash", "space", "ssharp", "sterling", "Tab", "Uacute", "underscore", "yen"
									};
	static char xmodmapASCIIequivalents[] = {
		231,229,174,190,38,39,129,140,94,126,42,64,92,124,123,125,91,93,130,141,162,58,44,169,219,160,250,251,171,249,255,252,246,172,253,96,248,254,247,214,36,224,227,201,209,208,61,33,193,96,62,199,200,234,235,236,245,220,210,60,188,45,181,194,35,238,239,206,207,241,175,187,191,166,40,41,37,46,225,43,177,63,192,34,168,13,211,213,164,59,226,47,32,167,163,9,242,95,180
									};
	int high = numberOfSymbols; //used for divide-and-conquer algorithm
	int low = 0;
	int testIndex;
	int comparisonResult;
	char asciiRepresentation = '\0';
	bool stopLooping = false;
		
	while(stopLooping == false){
		testIndex = (high+low)/2;
		comparisonResult = stringCompare(xmodmapStrings[testIndex], stringToResolve);
		if(comparisonResult > 0) //our guess comes after our goal
			high = testIndex;
		else if(comparisonResult < 0)
			low = testIndex;
		else{
			//we found it!
			asciiRepresentation = xmodmapASCIIequivalents[testIndex];
			stopLooping = true;
		}
		if( (high - low) == 1) //it isn't there.
			stopLooping = true;
	}
	return asciiRepresentation;
}


/**********************************************
 * isCapsLockDown
 *	returns the state of the capsLock key
 *	true means it is on
 **********************************************/
bool KeyboardLayout::isCapsLockDown(){
	return capsLockDown;
}


/**********************************************
 * setCapsLock
 *	sets the state of the capsLock key
 *	true means it is on
 **********************************************/
void KeyboardLayout::setCapsLock(bool newState){
	capsLockDown = newState;
}