/*
 *  standardFunctions.c
 *  Keymileage
 *
 *	Library of functions that come in handy
 *
 *  Created by Allen Smith on Tue Feb 24 2004.
 *
 */

#include "standardFunctions.h"

#include <stdlib.h>
#include <string.h>
#include <ctype.h>

/**********************************************
 * parseFloat
 *	accepts a string of characters and reads a float from it
 *	differs from atof in that the first character need not be numeric
 **********************************************/
float parseFloat(char *inputString){
	return atof(inputString + strcspn( inputString, "0123456789" ));
}


/**********************************************
 * parseInt
 *	accepts a string of characters and reads a integer from it
 *	differs from atoi in that the first character need not be numeric
 **********************************************/
int parseInt(char *inputString){
	return atoi(inputString + strcspn( inputString, "0123456789" ));
}

/**********************************************
 * stringCompare
 *	replacement to strcmp which compares strings lexographically
 *	capitals preceed lowercase
 *	"this" preceeds "thisword"
 *	returns
 *		0 if strings are identical
 *		positive if string1 is greater than (proceeds) string2
 *		negative if string1 is less than (preceeds) string2
 **********************************************/
int stringCompare(char *string1, char *string2){
	int comparator = 0;
	int counter = 0;
	char comparingLetters[2] = {1,1}; //just temporary to prevent it from stopping early
		
	//only bother if we are dealing with strings longer than 0
	//or as long as these strings *aren't* actually equal
	if(strlen(string1) > 0 &&
	   strlen(string2) > 0 &&
	   strcmp(string1, string2) != 0){
	   
		//keep comparing as long as necessary to find either a difference or the end of the string!
		while(comparator == 0 && comparingLetters[0] != '\0'){
			//first compare on lower-case:
			comparingLetters[0] = tolower(string1[counter]);
			comparingLetters[1] = tolower(string2[counter]);
			
			if(comparingLetters[0] != comparingLetters[1])
				if(comparingLetters[0] > comparingLetters[1] )
					comparator = 1; //string1 lexigraphical proceeds string2
				else
					comparator = -1;//string1 lexigraphical preceeds string2
			else
				counter++;
		}
		if(comparator == 0)
			comparator = strcmp(string1, string2); //if the strings are identical as lower-case, then they must be compared normally
	} //end if minimumLength > 0
	
	return comparator;
}

